<?php
/*
Plugin Name: physical2virtual
Version: 2.6.i
Description: Autoconvert physical albums to virtual
Plugin URI: http://piwigo.org/ext/extension_view.php?eid=730
Author: JanisV
*/

if (!defined('PHPWG_ROOT_PATH')) die('Hacking attempt!');

global $prefixeTable; 

define('PHYSICAL2VIRTUAL_ID',       basename(dirname(__FILE__)));
define('PHYSICAL2VIRTUAL_PATH' ,    PHPWG_PLUGINS_PATH . PHYSICAL2VIRTUAL_ID . '/');
define('PHYSICAL2VIRTUAL_ADMIN',    get_root_url() . 'admin.php?page=plugin-' . PHYSICAL2VIRTUAL_ID);
define('PHY2VIRT_CATEGORIES_TABLE', $prefixeTable . 'phy2virt_categories');

add_event_handler('init', 'physical2virtual_init');
if (defined('IN_ADMIN'))
{
  add_event_handler('get_admin_plugin_menu_links', 'physical2virtual_admin_menu');
} 

function physical2virtual_init()
{
  global $conf;
  
  load_language('plugin.lang', PHYSICAL2VIRTUAL_PATH);

  $conf['physical2virtual'] = unserialize($conf['physical2virtual']);

//  add_event_handler('invalidate_user_cache', 'physical2virtual_convert');
}

function find_or_create_virtual_category($phy_category, $parent_id=null, $options=array())
{
  $query = '
SELECT DISTINCT virt_category_id AS id, representative_picture_id
  FROM '.PHY2VIRT_CATEGORIES_TABLE.'
  JOIN '.CATEGORIES_TABLE.' ON '.PHY2VIRT_CATEGORIES_TABLE.'.virt_category_id = '.CATEGORIES_TABLE.'.id
  WHERE phy_category_id = '.$phy_category['id'].'
;';
  $virt_category = pwg_db_fetch_assoc(pwg_query($query));
  if (!empty($virt_category))
  {
    return $virt_category;
  }
  
  $category_name = $phy_category['name'];
  // process empty album name
  if (preg_match('/^\s*$/', $category_name))
  {
    $category_name = '_';
  }

  if (!is_null($parent_id))
  {
    $sub_ids = get_subcat_ids(array($parent_id));
    foreach ($sub_ids as $id)
    {
      $cat_info = get_cat_info($id);
      if ($cat_info['name'] == $category_name and !$cat_info['dir'])
        return $cat_info;
    }
  }

  $cat = create_virtual_category($category_name, $parent_id, $options);
  $insert = array(
    'phy_category_id' => $phy_category['id'],
    'virt_category_id' => $cat['id'],
//    'updated' => date('Y-m-d H:i:s'),
    );
  single_insert(PHY2VIRT_CATEGORIES_TABLE, $insert);

  return $cat;
}

function physical2virtual_convert()
{
  global $conf;
  
  $options=array();
  if (isset($conf['physical2virtual']['unlock_virtual']) and $conf['physical2virtual']['unlock_virtual'])
    $options['visible'] = true;
  $options['inherit'] = $conf['physical2virtual']['inherit'];

  // Remove virtual without physical
  $query = '
    SELECT virt_category_id
    FROM '.PHY2VIRT_CATEGORIES_TABLE.'
    WHERE phy_category_id NOT IN (
      SELECT cat.id
      FROM '.CATEGORIES_TABLE.' cat
      WHERE cat.dir IS NOT NULL
    )
  ;';
  $category_ids = array_from_query($query, 'virt_category_id');
  delete_categories($category_ids);

  $query = '
    DELETE FROM '.PHY2VIRT_CATEGORIES_TABLE.'
    WHERE phy_category_id NOT IN (
      SELECT cat.id
      FROM '.CATEGORIES_TABLE.' cat
      WHERE cat.dir IS NOT NULL
    )
  ;';
  pwg_query($query);
  $query = '
    DELETE FROM '.PHY2VIRT_CATEGORIES_TABLE.'
    WHERE virt_category_id NOT IN (
      SELECT cat.id
      FROM '.CATEGORIES_TABLE.' cat
      WHERE cat.dir IS NOT NULL
    )
  ;';
  pwg_query($query);
  
  $query = '
    SELECT id, name, id_uppercat
    FROM '.CATEGORIES_TABLE.'
    WHERE dir IS NOT NULL
  ;';  // get physical
  $result = pwg_query($query);
  while ($row = pwg_db_fetch_assoc($result))
  {
    $parent = isset($conf['physical2virtual']['parent_cat']) ? $conf['physical2virtual']['parent_cat'] : null;
        
    if ($conf['physical2virtual']['store_structure'])
    {
      // get array of physical parents
      $cat_info = get_cat_info($row['id']);
      $uppers = $cat_info['upper_names'];
      array_pop($uppers); // remove themself
      
      foreach ($uppers as $upper)
      {
        $t = find_or_create_virtual_category($upper, $parent, $options);
        $parent = $t['id'];
      }
    }

    $virt_cat = find_or_create_virtual_category($row, $parent, $options);

    if (isset($conf['physical2virtual']['lock_physical']) and $conf['physical2virtual']['lock_physical'])
    {
      $cat_info = get_cat_info($row['id']);
      if ($cat_info['visible'] == true)
      {
        set_cat_visible(array($row['id']), false);
      }
    }
    if (isset($conf['physical2virtual']['private_physical']) and $conf['physical2virtual']['private_physical'])
    {
      set_cat_status(array($row['id']), 'private'); 
    }

    $query = '
      INSERT IGNORE INTO '.IMAGE_CATEGORY_TABLE.' (category_id, image_id)
        SELECT '.$virt_cat['id'].', image_id FROM '.IMAGE_CATEGORY_TABLE.' WHERE category_id = '.$row['id'].'
    ;';
    pwg_query($query);

    $query = '
UPDATE '.PHY2VIRT_CATEGORIES_TABLE.'
  SET updated = NOW()
  WHERE virt_category_id = '.$virt_cat['id'].'
    ;';
    pwg_query($query);

    if (!(isset($virt_cat['representative_picture_id']) and $virt_cat['representative_picture_id']))
    {
      set_random_representant(array($virt_cat['id']));
    }
  }
}

function physical2virtual_admin_menu($menu)
{
  $menu[] = array(
    'NAME' => 'physical2virtual',
    'URL'  => PHYSICAL2VIRTUAL_ADMIN,
  );

  return $menu;
}

?>
